#define SC_USE_USER_DEFINED_VS_MAIN

#include <std.glsl>
#include <std_vs.glsl>
#include <std_fs.glsl>

uniform vec4 baseColor;

uniform sampler2D baseTex;
#if !defined(NOMASK)
uniform sampler2D opacityTex;
#endif

#ifdef VERTEX_SHADER

void main(void){
	sc_Vertex_t v=sc_LoadVertexAttributes();
	sc_ProcessVertex(v);
}

#endif

#ifdef FRAGMENT_SHADER

vec3 mapColor(vec3 orgColor)
{
	const float EPS=0.000001;
	const float pxSize=512.0;

	float bValue=(orgColor.b*255.0)/4.0;

	vec2 mulB=clamp(floor(bValue)+vec2(0.0,1.0),0.0,63.0);
	vec2 row=floor(mulB/8.0+EPS);
	vec4 row_col=vec4(row,mulB -row*8.0);
	const vec4 offset=vec4(1.0 -0.5/pxSize,1.0 -0.5/pxSize,-0.5/pxSize,-0.5/pxSize);
	vec4 lookup=orgColor.ggrr*(63.0/pxSize)+row_col*(64.0/pxSize)+(0.5/pxSize);
	float b1w=bValue -mulB.x;

	vec3 sampled1=texture2D(baseTex,vec2(lookup.z,1.0 -lookup.x)).rgb;
	vec3 sampled2=texture2D(baseTex,vec2(lookup.w,1.0 -lookup.y)).rgb;

	vec3 res=mix(sampled1,sampled2,b1w);
	return res;
}

void main(void){
	vec3 originalColor=getFramebufferColor().rgb;
	vec3 color=mapColor(originalColor)*baseColor.rgb;

#if defined(NOMASK)
	float factor=baseColor.a;
#else
	float factor=texture2D(opacityTex,varTex0).r*baseColor.a;
#endif

	gl_FragColor=vec4(mix(originalColor,color,factor),1.0);
}

#endif

#if sc_IsEditor
#error This is an exported shader. Please do not use shaders in Studio that have already been exported to a lens! Only use fresh shaders,presets,or shaders from existing Studio projects!
#endif
